//
//                                                                         
// Module: Internals Example Source File                                   
//                                                                         
// Description: Declarations for the Internals Example Plugin              
//                                                                         
//                                                                         
// This source code module, and all information, data, and algorithms      
// associated with it, are part of CUBE technology (tm).                   
//                 PROPRIETARY AND CONFIDENTIAL                            
// Copyright (c) 1996-2008 Image Space Incorporated.  All rights reserved. 
//                                                                         
//                                                                         
// Change history:                                                         
//   tag.2005.11.30: created                                               
//                                                                         
//

#include "Example.hpp"          // corresponding header file
#include <math.h>               // for atan2, sqrt
#include <stdio.h>              // for sample output
#include <winbase.h>            // -- added by me

// plugin information

extern "C" __declspec( dllexport )
const char * __cdecl GetPluginName()                   { return( "VX Scoring for rFactor2" ); }

extern "C" __declspec( dllexport )
PluginObjectType __cdecl GetPluginType()               { return( PO_INTERNALS ); }

extern "C" __declspec( dllexport )
int __cdecl GetPluginVersion()                         { return( 7 ); } // InternalsPluginV07 functionality

extern "C" __declspec( dllexport )
PluginObject * __cdecl CreatePluginObject()            { return( (PluginObject *) new ExampleInternalsPlugin ); }

extern "C" __declspec( dllexport )
void __cdecl DestroyPluginObject( PluginObject *obj )  { delete( (ExampleInternalsPlugin *) obj ); }

// -- ### my coding starts here ###
char globalLogTime[32]                = "0";
char globalServerStartTime[32]        = "0";
char globalSessionStartTime[32]       = "0";
char globalGameBuild[12]              = "1.1098";
char globalGameVersion[64]            = "rFactor2 Build: 1.1098";
char globalPluginVersion[64]          = "VX Scoring2 Build: 1.1098.7.20161211";   // -- Build: {rFactorBuildNo}.{InternalsPluginVersion}.{CompileDate}

char globalCurrentDirectory[1024]     = "0";
char globalConfigIni[1024]            = "0";

char globalPlrFileName[128]           = "";
char globalLogFile[2048]              = "VXScoring\\VXScoring.log";
char globalLogMsg[2048]               = "0";
char globalScoringTempFile[2048]      = "VXScoring\\Scoring.tmp~";
char globalScoringDataFile[2048]      = "VXScoring\\Scoring.xml";
char globalResultStreamFile[2048]     = "VXScoring\\Stream.xml";
char globalResultStreamFiltered[2048] = "VXScoring\\StreamFiltered.xml";

int globalInGameStart                 = 0;
int globalScoringOutput               = 0;
int globalUseSubfolders               = 0;

long globalSessionID                  = 0;

void SystemTimePush(char TimeArray[]){
   try{
      SYSTEMTIME st;

      GetLocalTime(&st);

      sprintf(TimeArray, "%04d-%02d-%02d-%02d-%02d-%02d", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
   }
   catch(...){
      // -- just catch
   }
}

void WriteLog(){
   try{
      FILE *fpLog = fopen(globalLogFile, "a");

      if(fpLog != NULL){
         SystemTimePush(globalLogTime);

         fprintf(fpLog, "%s: %s\n", globalLogTime, globalLogMsg);

         fflush(fpLog);
         fclose(fpLog);

         fpLog = NULL;
      }
         sprintf(globalLogMsg, "0");
      }
   catch(...){
      // -- just catch
   }
}

int GetInGameStart(){
   try{
      switch(globalSessionID){
          case 0: globalInGameStart = GetPrivateProfileIntA("InGameStart", "TestDay",     -10, globalConfigIni); break; // -- Test Day
          case 1: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Practice1",   -11, globalConfigIni); break; // -- Practice 1
          case 2: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Practice2",   -12, globalConfigIni); break; // -- Practice 2
          case 3: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Practice3",   -13, globalConfigIni); break; // -- Practice 3
          case 4: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Practice4",   -14, globalConfigIni); break; // -- Practice 4
          case 5: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Qualifying1", -15, globalConfigIni); break; // -- Qualifying 1
          case 6: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Qualifying2", -16, globalConfigIni); break; // -- Qualifying 2
          case 7: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Qualifying3", -17, globalConfigIni); break; // -- Qualifying 3
          case 8: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Qualifying4", -18, globalConfigIni); break; // -- Qualifying 4
          case 9: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Warmup",      -19, globalConfigIni); break; // -- Warmup
         case 10: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Race1",       -20, globalConfigIni); break; // -- Race 1
         case 11: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Race2",       -21, globalConfigIni); break; // -- Race 2
         case 12: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Race3",       -22, globalConfigIni); break; // -- Race 3
         case 13: globalInGameStart = GetPrivateProfileIntA("InGameStart", "Race4",       -23, globalConfigIni); break; // -- Race 4
         default: globalInGameStart = -24; break;
      }
   }catch(...){
      globalInGameStart = -25;
   }
   return(globalInGameStart);
}

bool WriteDefaultSessionFiles(int ServerState){
   if(globalScoringOutput == 1){
      try{
         FILE *fpScore = fopen(globalScoringDataFile, "w");

         if(fpScore != NULL){
            fprintf(fpScore, "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\n");
            fprintf(fpScore, "<rFactorScoring>\n");
            fprintf(fpScore, "<Copyright>VX Racing</Copyright>\n");
            fprintf(fpScore, "<SessionInfo>\n");
            fprintf(fpScore, " <rFactorID>2</rFactorID>\n");
            fprintf(fpScore, " <rFactorVersion>%s</rFactorVersion>\n", globalGameVersion);
            fprintf(fpScore, " <PluginVersion>%s</PluginVersion>\n", globalPluginVersion);
            fprintf(fpScore, " <ServerStart>%s</ServerStart>\n", globalServerStartTime);
            fprintf(fpScore, " <ServerState>%d</ServerState>\n", ServerState);
            fprintf(fpScore, " <SessionStart>0</SessionStart>\n");
            fprintf(fpScore, "</SessionInfo>\n");
            fprintf(fpScore, "</rFactorScoring>\n");

            fflush(fpScore);
            fclose(fpScore);

            fpScore = NULL;
         }
      }catch(...){
         return(false);
      }

      try{
         FILE *fpResult1 = fopen(globalResultStreamFile, "w");

         if(fpResult1 != NULL){
            fprintf(fpResult1, "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\n");
            fprintf(fpResult1, "<rFactorStream>\n");
            fprintf(fpResult1, "<Copyright>VX Racing</Copyright>\n");
            fprintf(fpResult1, "<SessionInfo>\n");
            fprintf(fpResult1, " <rFactorID>2</rFactorID>\n");
            fprintf(fpResult1, " <rFactorVersion>%s</rFactorVersion>\n", globalGameVersion);
            fprintf(fpResult1, " <PluginVersion>%s</PluginVersion>\n", globalPluginVersion);
            fprintf(fpResult1, " <ServerStart>%s</ServerStart>\n", globalServerStartTime);
            fprintf(fpResult1, " <ServerState>%d</ServerState>\n", ServerState);
            fprintf(fpResult1, " <SessionStart>0</SessionStart>\n");
            fprintf(fpResult1, "</SessionInfo>\n");
            fprintf(fpResult1, "</rFactorStream>\n");

            fflush(fpResult1);
            fclose(fpResult1);

            fpResult1 = NULL;
         }
      }catch(...){
         return(false);
      }
   }

   return(true);
}

bool WriteStreamFile(){
   if(globalScoringOutput == 1){
      try{
         FILE *fpResult2 = fopen(globalResultStreamFile, "w");

         if(fpResult2 != NULL){
            fprintf(fpResult2, "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\n");
            fprintf(fpResult2, "<rFactorStream>\n");
            fprintf(fpResult2, "<Copyright>VX Racing</Copyright>\n");
            fprintf(fpResult2, "<SessionInfo>\n");
            fprintf(fpResult2, " <rFactorID>2</rFactorID>\n");
            fprintf(fpResult2, " <rFactorVersion>%s</rFactorVersion>\n", globalGameVersion);
            fprintf(fpResult2, " <PluginVersion>%s</PluginVersion>\n", globalPluginVersion);
            fprintf(fpResult2, " <ServerStart>%s</ServerStart>\n", globalServerStartTime);
            fprintf(fpResult2, " <ServerState>2</ServerState>\n");
            fprintf(fpResult2, " <SessionStart>%s</SessionStart>\n", globalSessionStartTime);
            fprintf(fpResult2, "</SessionInfo>\n");
            fprintf(fpResult2, "<Stream>\n");
            fprintf(fpResult2, "<Plugin et=\"0.0\">New session started</Plugin>\n");
            fprintf(fpResult2, "<Plugin et=\"0.1\">Result stream output is enabled for all nodes</Plugin>\n");

            fflush(fpResult2);
            fclose(fpResult2);

            fpResult2 = NULL;
         }
      }catch(...){
         return(false);
      }

      try{
         FILE *fpResult3 = fopen(globalResultStreamFiltered, "w");

         if(fpResult3 != NULL){
            fprintf(fpResult3, "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\n");
            fprintf(fpResult3, "<rFactorStream>\n");
            fprintf(fpResult3, "<Copyright>VX Racing</Copyright>\n");
            fprintf(fpResult3, "<SessionInfo>\n");
            fprintf(fpResult3, " <rFactorID>2</rFactorID>\n");
            fprintf(fpResult3, " <rFactorVersion>%s</rFactorVersion>\n", globalGameVersion);
            fprintf(fpResult3, " <PluginVersion>%s</PluginVersion>\n", globalPluginVersion);
            fprintf(fpResult3, " <ServerStart>%s</ServerStart>\n", globalServerStartTime);
            fprintf(fpResult3, " <ServerState>2</ServerState>\n");
            fprintf(fpResult3, " <SessionStart>%s</SessionStart>\n", globalSessionStartTime);
            fprintf(fpResult3, "</SessionInfo>\n");
            fprintf(fpResult3, "<Stream>\n");
            fprintf(fpResult3, "<Plugin et=\"0.0\">New session started</Plugin>\n");
            fprintf(fpResult3, "<Plugin et=\"0.1\">Result stream output is enabled for all but for score nodes</Plugin>\n");

            fflush(fpResult3);
            fclose(fpResult3);

            fpResult3 = NULL;
         }
      }catch(...){
         return(false);
      }
   }

   return(true);
}

bool FilterResultStream(char StreamData[]){
   try{
      char * tokStreamData = NULL;

      tokStreamData = strtok(StreamData, "\n");

      FILE *fpResult4 = fopen(globalResultStreamFiltered, "a");

      if(fpResult4 != NULL){
         while(tokStreamData != NULL){
            // -- if(strstr(tokStreamData, "<Chat") != NULL || strstr(tokStreamData, "<Incident") != NULL){

            if(strstr(tokStreamData, "<Score") == NULL){
               fprintf(fpResult4, "%s\n", tokStreamData);
            }

            tokStreamData = strtok(NULL, "\n");
         }

         fflush(fpResult4);
         fclose(fpResult4);

         fpResult4 = NULL;
      }

      sprintf(StreamData, "0");

      return(true);
   }catch(...){
      sprintf(globalLogMsg, "General exception FilterResultStream().");

      WriteLog();

      return(false);
   }
}

int InitScoringOutput(){
   try{
      if(globalUseSubfolders == 1){
         if(strlen(globalPlrFileName) > 0){
            char ScoringSubfolder[128] = "0";

            sprintf(ScoringSubfolder, "VXScoring\\%s", globalPlrFileName);

            if(CreateDirectory(ScoringSubfolder, NULL) != 0){
               sprintf(globalLogMsg, "Creating scoring subfolder '..\\%s'.", ScoringSubfolder);

               WriteLog();
            }
            
            sprintf(globalScoringTempFile, "%s\\VXScoring\\%s\\Scoring.tmp~", globalCurrentDirectory, globalPlrFileName);
            sprintf(globalLogMsg, "Using scoring temp file '%s'.", globalScoringTempFile);

            WriteLog();

            sprintf(globalScoringDataFile, "%s\\VXScoring\\%s\\Scoring.xml", globalCurrentDirectory, globalPlrFileName);
            sprintf(globalLogMsg, "Using scoring data file '%s'.", globalScoringDataFile);

            WriteLog();

            sprintf(globalResultStreamFile, "%s\\VXScoring\\%s\\Stream.xml", globalCurrentDirectory, globalPlrFileName);
            sprintf(globalLogMsg, "Using stream data file '%s'.", globalResultStreamFile);

            WriteLog();
            
            sprintf(globalResultStreamFiltered, "%s\\VXScoring\\%s\\StreamFiltered.xml", globalCurrentDirectory, globalPlrFileName);
            sprintf(globalLogMsg, "Using filtered stream data file '%s'.", globalResultStreamFiltered);

            WriteLog();

            globalScoringOutput = 1;

            // -- clean scoring data in ..\VXScoring root folder, if it was formerly generated without subfolders
            if(remove("VXScoring\\Scoring.tmp~") != 0){
               // -- just catch
            }

            if(remove("VXScoring\\Scoring.xml") != 0){
               // -- just catch
            }

            if(remove("VXScoring\\Stream.xml") != 0){
               // -- just catch
            }
            
            if(remove("VXScoring\\StreamFiltered.xml") != 0){
               // -- just catch
            }
         }
      }else{
         globalScoringOutput = 1;
      }
   }catch(...){
      sprintf(globalLogMsg, "General exception InitScoringOutput().");

      WriteLog();

      return(1);
   }
   return(0);
}

// -- ExampleInternalsPlugin class

void ExampleInternalsPlugin::WriteToAllExampleOutputFiles(const char * const openStr, const char * const msg){
   // -- not required in this plugin
}

void ExampleInternalsPlugin::Startup(long version){
   // -- Open ports, read configs, whatever you need to do.

   if(CreateDirectory("VXScoring", NULL) == 0){
      // -- just catch
   }

   sprintf(globalLogMsg, "Server is starting up with plugin '%s' ...", globalPluginVersion);

   WriteLog();

   SystemTimePush(globalServerStartTime);

   sprintf(globalLogMsg, "New server start time is '%s'.", globalServerStartTime);

   WriteLog();

   if(version > 10000){
      float fBuildVersion = version / 10000.0f;

      sprintf(globalGameBuild, "%.4f", fBuildVersion);
   }

   sprintf(globalGameVersion, "rFactor2 Build: %s", globalGameBuild);

   GetCurrentDirectoryA(sizeof(globalCurrentDirectory), globalCurrentDirectory);
   sprintf(globalLogMsg, "Current directory given is '%s'.", globalCurrentDirectory);

   WriteLog();

   sprintf(globalConfigIni, "%s\\VXScoring.ini", globalCurrentDirectory);
   sprintf(globalLogMsg, "Using plugin ini file '%s'.", globalConfigIni);

   WriteLog();

   sprintf(globalLogFile, "%s\\VXScoring\\VXScoring.log", globalCurrentDirectory);
   sprintf(globalLogMsg, "Using log file '%s'.", globalLogFile);

   WriteLog();

   sprintf(globalScoringTempFile, "%s\\VXScoring\\Scoring.tmp~", globalCurrentDirectory);
   sprintf(globalLogMsg, "Using scoring temp file '%s'.", globalScoringTempFile);

   WriteLog();

   sprintf(globalScoringDataFile, "%s\\VXScoring\\Scoring.xml", globalCurrentDirectory);
   sprintf(globalLogMsg, "Using scoring data file '%s'.", globalScoringDataFile);

   WriteLog();

   sprintf(globalResultStreamFile, "%s\\VXScoring\\Stream.xml", globalCurrentDirectory);
   sprintf(globalLogMsg, "Using result stream file '%s'.", globalResultStreamFile);

   WriteLog();

   globalUseSubfolders = GetPrivateProfileIntA("DataOutput", "UseSubfolders", 0, globalConfigIni);

   InitScoringOutput();
   
   if(WriteDefaultSessionFiles(1) == false){
      WriteDefaultSessionFiles(1);
   }
   
   // -- Default HW control. Enabled to true.
   mEnabled = false;
}

void ExampleInternalsPlugin::Shutdown(){
   sprintf(globalLogMsg, "Server is shuting down ...");

   WriteLog();

   sprintf(globalServerStartTime, "0");

   sprintf(globalLogMsg, "New server start time is '%s'.", globalServerStartTime);

   WriteLog();

   InitScoringOutput();

   if(WriteDefaultSessionFiles(0) == false){
      WriteDefaultSessionFiles(0);
   }
}

void ExampleInternalsPlugin::StartSession(){
   sprintf(globalLogMsg, "Server is starting new session ...");

   WriteLog();

   SystemTimePush(globalSessionStartTime);

   sprintf(globalLogMsg, "New session start time is '%s'.", globalSessionStartTime);

   WriteLog();

   InitScoringOutput();

   if(WriteStreamFile() == false){
      WriteStreamFile();
   }
}

void ExampleInternalsPlugin::EndSession(){
   sprintf(globalLogMsg, "Server is ending current session ...");

   WriteLog();

   InitScoringOutput();

   if(WriteDefaultSessionFiles(3) == false){
      WriteDefaultSessionFiles(3);
   }
}

void ExampleInternalsPlugin::EnterRealtime(){
   sprintf(globalLogMsg, "Server is entering realtime ...");

   WriteLog();
   
   // -- start up timer every time we enter realtime
   mET = 0.0f;
}

void ExampleInternalsPlugin::ExitRealtime(){
   // -- not required in this plugin

   sprintf(globalLogMsg, "Server is exiting realtime ...");

   WriteLog();
}

void ExampleInternalsPlugin::UpdateTelemetry(const TelemInfoV01 &info){
   // -- not required in this plugin

   sprintf(globalLogMsg, "Server updates telemetry ...");

   WriteLog();
}

void ExampleInternalsPlugin::UpdateGraphics(const GraphicsInfoV01 &info){
   // -- not required in this plugin

   sprintf(globalLogMsg, "Server updates graphics ...");

   WriteLog();
}

bool ExampleInternalsPlugin::CheckHWControl(const char * const controlName, double &fRetVal){
   // -- not required in this plugin

   sprintf(globalLogMsg, "Server checks hw controls ...");

   WriteLog();

   return(false);
}

bool ExampleInternalsPlugin::ForceFeedback(double &forceValue){
   // -- not required in this plugin
   
   sprintf(globalLogMsg, "Server handels force feedback ...");

   WriteLog();

   return(false);
}

void ExampleInternalsPlugin::UpdateScoring(const ScoringInfoV01 &info){
   // -- note: function is called twice (5 times ???) per second now (instead of once per second in previous versions)
   
   // -- always asign the session number to a global int variable
   globalSessionID = info.mSession;

   // -- get in game starting time from plugin ini file
   try{
      globalInGameStart = GetInGameStart();
   }
   catch(...){
      globalInGameStart = -26;
   }

   // -- save PlrFileName into glabal variable for further use
   try{
      if(strlen(info.mPlrFileName) > 0 && strlen(globalPlrFileName) == 0){
         sprintf(globalPlrFileName, "%s", info.mPlrFileName);

         sprintf(globalLogMsg, "Using PlrFileName '%s'.", globalPlrFileName);

         WriteLog();

         InitScoringOutput();

         if(WriteStreamFile() == false){
            WriteStreamFile();
         }
      }
   }catch(...){
      // -- just catch
   }
      
   // -- print current scoring to file
   try{
      FILE *fpScoreTemp = fopen(globalScoringTempFile, "w");

      if(fpScoreTemp != NULL){
         // -- print xml header and session info
         fprintf(fpScoreTemp, "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\n");
         fprintf(fpScoreTemp, "<\?xml-stylesheet type=\"text/xsl\" href=\"Scoring.xsl\"\?>\n");
         fprintf(fpScoreTemp, "<rFactorScoring>\n");
         fprintf(fpScoreTemp, "<Copyright>VX Racing</Copyright>\n");
         fprintf(fpScoreTemp, "<SessionInfo>\n");
         fprintf(fpScoreTemp, " <rFactorID>2</rFactorID>\n");
         fprintf(fpScoreTemp, " <rFactorVersion>%s</rFactorVersion>\n", globalGameVersion);
         fprintf(fpScoreTemp, " <PluginVersion>%s</PluginVersion>\n", globalPluginVersion);
         fprintf(fpScoreTemp, " <ServerStart>%s</ServerStart>\n", globalServerStartTime);
         fprintf(fpScoreTemp, " <ServerState>2</ServerState>\n");
         fprintf(fpScoreTemp, " <SessionStart>%s</SessionStart>\n", globalSessionStartTime);
         fprintf(fpScoreTemp, " <SessionID>%d</SessionID>\n", info.mSession);
         fprintf(fpScoreTemp, " <SessionState>%d</SessionState>\n", info.mGamePhase);
         fprintf(fpScoreTemp, " <SessionTime>%.1f</SessionTime>\n", info.mCurrentET);
         fprintf(fpScoreTemp, " <SessionEnd>%.1f</SessionEnd>\n", info.mEndET);
         fprintf(fpScoreTemp, " <MaxLaps>%d</MaxLaps>\n", info.mMaxLaps);
         fprintf(fpScoreTemp, " <TrackName>%s</TrackName>\n", info.mTrackName);
         fprintf(fpScoreTemp, " <TrackLength>%.1f</TrackLength>\n", info.mLapDist);
         fprintf(fpScoreTemp, " <NumVehicles>%d</NumVehicles>\n", info.mNumVehicles);
         fprintf(fpScoreTemp, " <InGameStart>%d</InGameStart>\n", globalInGameStart);
         fprintf(fpScoreTemp, " <YellowFlagState>%d</YellowFlagState>\n", info.mYellowFlagState);
         fprintf(fpScoreTemp, " <SectorFlags>%d::%d::%d</SectorFlags>\n", info.mSectorFlag[2], info.mSectorFlag[0], info.mSectorFlag[1]); // -- [2]==1st [0]==2nd [1]==3rd
         fprintf(fpScoreTemp, " <InRealtime>%d</InRealtime>\n", info.mInRealtime);
         fprintf(fpScoreTemp, " <StartLight>%d</StartLight>\n", info.mStartLight);
         fprintf(fpScoreTemp, " <NumRedLights>%d</NumRedLights>\n", info.mNumRedLights);
         fprintf(fpScoreTemp, " <PlayerName>%s</PlayerName>\n", info.mPlayerName);
         fprintf(fpScoreTemp, " <PlrFileName>%s</PlrFileName>\n", info.mPlrFileName);
         fprintf(fpScoreTemp, " <DarkCloud>%.2f</DarkCloud>\n", info.mDarkCloud);
         fprintf(fpScoreTemp, " <Raining>%.2f</Raining>\n", info.mRaining);
         fprintf(fpScoreTemp, " <AmbientTemp>%.1f</AmbientTemp>\n", info.mAmbientTemp);
         fprintf(fpScoreTemp, " <TrackTemp>%.1f</TrackTemp>\n", info.mTrackTemp);
         fprintf(fpScoreTemp, " <Wind>%.1f::%.1f::%.1f</Wind>\n", info.mWind.x, info.mWind.y, info.mWind.z);
         fprintf(fpScoreTemp, " <OnPathWetness>%.2f</OnPathWetness>\n", info.mMaxPathWetness);
         fprintf(fpScoreTemp, " <OffPathWetness>%.2f</OffPathWetness>\n", info.mMinPathWetness);
         fprintf(fpScoreTemp, "</SessionInfo>\n");
         
         // -- print slot info
         fprintf(fpScoreTemp, "<Slots>\n");
         for(long i = 0; i < info.mNumVehicles; ++i){
            VehicleScoringInfoV01 &vinfo = info.mVehicle[i];
            
            double VehicleSpeed = fabs(vinfo.mLocalVel.z);
            
            fprintf(fpScoreTemp, "<Slot>\n");
            fprintf(fpScoreTemp, " <SlotID>%d</SlotID>\n", i);
            fprintf(fpScoreTemp, " <SlotSID>%d</SlotSID>\n", vinfo.mID);
            fprintf(fpScoreTemp, " <Place>%d</Place>\n", vinfo.mPlace);
            fprintf(fpScoreTemp, " <DriverName>%s</DriverName>\n", vinfo.mDriverName);
            fprintf(fpScoreTemp, " <VehicleClass>%s</VehicleClass>\n", vinfo.mVehicleClass);
            fprintf(fpScoreTemp, " <VehicleName>%s</VehicleName>\n", vinfo.mVehicleName);
            fprintf(fpScoreTemp, " <Control>%d</Control>\n", vinfo.mControl);
            fprintf(fpScoreTemp, " <Laps>%d</Laps>\n", vinfo.mTotalLaps);
            fprintf(fpScoreTemp, " <LapStart>%.3f</LapStart>\n", vinfo.mLapStartET);
            fprintf(fpScoreTemp, " <LapDist>%.1f</LapDist>\n", vinfo.mLapDist);
            fprintf(fpScoreTemp, " <Sector>%d</Sector>\n", vinfo.mSector);
            fprintf(fpScoreTemp, " <CurS1>%.3f</CurS1>\n", vinfo.mCurSector1);
            fprintf(fpScoreTemp, " <CurS2>%.3f</CurS2>\n", vinfo.mCurSector2);
            fprintf(fpScoreTemp, " <LastS1>%.3f</LastS1>\n", vinfo.mLastSector1);
            fprintf(fpScoreTemp, " <LastS2>%.3f</LastS2>\n", vinfo.mLastSector2);
            fprintf(fpScoreTemp, " <LastS3>%.3f</LastS3>\n", vinfo.mLastLapTime);
            fprintf(fpScoreTemp, " <LastLap>%.3f</LastLap>\n", vinfo.mLastLapTime);
            fprintf(fpScoreTemp, " <BestS1>%.3f</BestS1>\n", vinfo.mBestSector1);
            fprintf(fpScoreTemp, " <BestS2>%.3f</BestS2>\n", vinfo.mBestSector2);
            fprintf(fpScoreTemp, " <BestS3>%.3f</BestS3>\n", vinfo.mBestLapTime);
            fprintf(fpScoreTemp, " <BestLap>%.3f</BestLap>\n", vinfo.mBestLapTime);
            fprintf(fpScoreTemp, " <GapTime>%.3f</GapTime>\n", vinfo.mTimeBehindLeader);
            fprintf(fpScoreTemp, " <GapLaps>%d</GapLaps>\n", vinfo.mLapsBehindLeader);
            fprintf(fpScoreTemp, " <IntTime>%.3f</IntTime>\n", vinfo.mTimeBehindNext);
            fprintf(fpScoreTemp, " <IntLaps>%d</IntLaps>\n", vinfo.mLapsBehindNext);
            fprintf(fpScoreTemp, " <InPits>%d</InPits>\n", vinfo.mInPits);
            fprintf(fpScoreTemp, " <PitState>%u</PitState>\n", vinfo.mPitState);
            fprintf(fpScoreTemp, " <PitGroup>%s</PitGroup>\n", vinfo.mPitGroup);
            fprintf(fpScoreTemp, " <InBox>%d</InBox>\n", vinfo.mInGarageStall);
            fprintf(fpScoreTemp, " <Pitstops>%d</Pitstops>\n", vinfo.mNumPitstops);
            fprintf(fpScoreTemp, " <Penalties>%d</Penalties>\n", vinfo.mNumPenalties);
            fprintf(fpScoreTemp, " <FinishStatus>%d</FinishStatus>\n", vinfo.mFinishStatus);
            fprintf(fpScoreTemp, " <VehicleSpeed>%.3f</VehicleSpeed>\n", VehicleSpeed);
            fprintf(fpScoreTemp, " <PosX>%.3f</PosX>\n", vinfo.mPos.x);
            fprintf(fpScoreTemp, " <PosY>%.3f</PosY>\n", vinfo.mPos.z);
            fprintf(fpScoreTemp, "</Slot>\n");
         }
         
         // -- close open xml nodes
         fprintf(fpScoreTemp, "</Slots>\n");
         fprintf(fpScoreTemp, "</rFactorScoring>\n");
         
         // -- close current scoring temp file
         fflush(fpScoreTemp);
         fclose(fpScoreTemp);

         fpScoreTemp = NULL;
         
         // -- transport temp data to prod data
         try{
            /*
            if(remove(globalScoringDataFile) != 0){
               sprintf(globalLogMsg, "Error deleting scoring data file.");
               WriteLog();
            }

            if(rename(globalScoringTempFile, globalScoringDataFile) != 0){
               sprintf(globalLogMsg, "Error renaming scoring temp file.");
               WriteLog();
            }
            */

            if(CopyFile(globalScoringTempFile, globalScoringDataFile, FALSE) == 0){
                sprintf(globalLogMsg, "Error copying scoring temp file to scoring data file.");

                WriteLog();
            }
         }catch(...){
            sprintf(globalLogMsg, "General exception while copying scoring temp data to scoring prod data.");

            WriteLog();
         }
      }
   }catch(...){
      sprintf(globalLogMsg, "General exception while writing scoring data.");

      WriteLog();
   }
   
   // -- put the current result stream in a separate file
   // -- note that only one plugin can use the stream (by enabling scoring updates) ... sorry if any clashes result
   try{
      char bufResultStream[32768] = "0";

      strcpy_s(bufResultStream, sizeof(bufResultStream), info.mResultsStream);

      if(strlen(bufResultStream) > 0){
         FILE *fpResult5 = fopen(globalResultStreamFile, "a");

         if(fpResult5 != NULL){
            fprintf(fpResult5, "%s", bufResultStream);

            fflush(fpResult5);
            fclose(fpResult5);

            fpResult5 = NULL;
         }

         FilterResultStream(bufResultStream);
      }

      sprintf(bufResultStream, "0");
   }catch(...){
      sprintf(globalLogMsg, "General exception in UpdateScoring().");

      WriteLog();
   }
}

bool ExampleInternalsPlugin::RequestCommentary(CommentaryRequestInfoV01 &info){
   // -- not required in this plugin

   return(false);
}
